
#pragma once

#include <cmath>
#include <cassert>
#include <mitsuba/core/matrix.h>
#include <mitsuba/core/vector.h>

namespace _HG_details {
    template <typename T>
    auto Power(T a, double e) { return pow(a,T(e)); }
    template <typename T>
    auto Sqrt(T a) { return sqrt(a); }
    static constexpr float Pi = M_PI;

    template <typename T>
    auto HG00(T x, T y, T z, T t11, T t12, T t13, T t22, T t23, T t33) {
        return 
            exp(-(Power(t23,2)*Power(x,2) + 2*t12*t33*x*y + Power(t13,2)*Power(y,2) - t11*t33*Power(y,2) - 2*t12*t13*y*z + Power(t12,2)*Power(z,2) - 2*t23*(t13*x*y + t12*x*z - t11*y*z) - t22*(t33*Power(x,2) - 2*t13*x*z + t11*Power(z,2)))/(2.*(Power(t13,2)*t22 - 2*t12*t13*t23 + Power(t12,2)*t33 + t11*(Power(t23,2) - t22*t33))));
    }

    template <typename T>
    auto HG20(T x, T y, T z, T t11, T t12, T t13, T t22, T t23, T t33) {
        return 
            exp(-(Power(t23,2)*Power(x,2) + 2*t12*t33*x*y + Power(t13,2)*Power(y,2) - t11*t33*Power(y,2) - 2*t12*t13*y*z + Power(t12,2)*Power(z,2) - 2*t23*(t13*x*y + t12*x*z - t11*y*z) - t22*(t33*Power(x,2) - 2*t13*x*z + t11*Power(z,2)))/(2.*(Power(t13,2)*t22 - 2*t12*t13*t23 + Power(t12,2)*t33 + t11*(Power(t23,2) - t22*t33))))
            *
            (2*Power(t12,2)*Power(t33,2)*Power(x,2) - Power(t13,4)*(t22 - 2*Power(y,2)) + t11*t12*t33*(t12*t33 - 4*t33*x*y + 4*t23*x*z) + 2*Power(t13,3)*(-2*t23*x*y + t12*(t23 - 2*y*z)) + Power(t13,2)*(2*Power(t23,2)*Power(x,2) + 4*t12*x*(t33*y + t23*z) - t11*(Power(t23,2) - 2*t22*t33 + 4*t33*Power(y,2) - 4*t23*y*z) - Power(t12,2)*(t33 - 2*Power(z,2))) + Power(t11,2)*(-(Power(t33,2)*(t22 - 2*Power(y,2))) - 4*t23*t33*y*z + Power(t23,2)*(t33 + 2*Power(z,2))) - 2*t13*(2*t12*t33*x*(t23*x + t12*z) + 2*t11*t23*x*(-(t33*y) + t23*z) + t11*t12*(-2*t33*y*z + t23*(t33 + 2*Power(z,2)))))
            /
            (Sqrt(2)*Power(Pi,0.75)*Power(-(Power(t13,2)*t22) + 2*t12*t13*t23 - t11*Power(t23,2) - Power(t12,2)*t33 + t11*t22*t33,0.25)*Power(Power(t13,2)*t22 - 2*t12*t13*t23 + Power(t12,2)*t33 + t11*(Power(t23,2) - t22*t33),2));
    }

    template <typename T>
    auto HG11(T x, T y, T z, T t11, T t12, T t13, T t22, T t23, T t33) {
        return 
            exp(-(Power(t23,2)*Power(x,2) + 2*t12*t33*x*y + Power(t13,2)*Power(y,2) - t11*t33*Power(y,2) - 2*t12*t13*y*z + Power(t12,2)*Power(z,2) - 2*t23*(t13*x*y + t12*x*z - t11*y*z) - t22*(t33*Power(x,2) - 2*t13*x*z + t11*Power(z,2)))/(2.*(Power(t13,2)*t22 - 2*t12*t13*t23 + Power(t12,2)*t33 + t11*(Power(t23,2) - t22*t33))))
            *
            (-(t12*t33*(Power(t12,2)*t33 - 2*(Power(t23,2) - t22*t33)*Power(x,2) + t12*(-2*t33*x*y + 2*t23*x*z))) + Power(t13,3)*(-2*t23*Power(y,2) + t22*(t23 + 2*y*z)) - Power(t13,2)*(2*x*(-2*Power(t23,2)*y + t22*t33*y + t22*t23*z) + t12*(2*Power(t23,2) + t22*t33 - 2*t33*Power(y,2) + 2*t22*Power(z,2))) + t11*(2*(Power(t23,2) - t22*t33)*x*(-(t33*y) + t23*z) + t12*(Power(t33,2)*(t22 - 2*Power(y,2)) + 4*t23*t33*y*z - Power(t23,2)*(t33 + 2*Power(z,2)))) + t13*(-2*t23*(Power(t23,2) - t22*t33)*Power(x,2) + 4*t12*t33*x*(-(t23*y) + t22*z) + Power(t12,2)*(3*t23*t33 - 2*t33*y*z + 2*t23*Power(z,2)) + t11*(Power(t23,3) - 2*Power(t23,2)*y*z - 2*t22*t33*y*z + t23*(-(t22*t33) + 2*t33*Power(y,2) + 2*t22*Power(z,2)))))
            /
            (Power(Pi,0.75)*Power(-(Power(t13,2)*t22) + 2*t12*t13*t23 - t11*Power(t23,2) - Power(t12,2)*t33 + t11*t22*t33,0.25)*Power(Power(t13,2)*t22 - 2*t12*t13*t23 + Power(t12,2)*t33 + t11*(Power(t23,2) - t22*t33),2));
    }

    template <typename T>
    auto HG31(T x, T y, T z, T t11, T t12, T t13, T t22, T t23, T t33) {
        return 
            -exp(-(Power(t23,2)*Power(x,2) + 2*t12*t33*x*y + Power(t13,2)*Power(y,2) - t11*t33*Power(y,2) - 2*t12*t13*y*z + Power(t12,2)*Power(z,2) - 2*t23*(t13*x*y + t12*x*z - t11*y*z) - t22*(t33*Power(x,2) - 2*t13*x*z + t11*Power(z,2)))/(2.*(Power(t13,2)*t22 - 2*t12*t13*t23 + Power(t12,2)*t33 + t11*(Power(t23,2) - t22*t33))))
            *
            ((3*(t13*t23 - t12*t33)*(Power(t13,2)*t22 - 2*t12*t13*t23 + Power(t12,2)*t33 + t11*(Power(t23,2) - t22*t33))*((Power(t13,2) - t11*t33)*(Power(t13,2)*t22 - 2*t12*t13*t23 + Power(t12,2)*t33 + t11*(Power(t23,2) - t22*t33)) - 2*Power(t12*t33*x + Power(t13,2)*y - t11*t33*y + t11*t23*z - t13*(t23*x + t12*z),2)) - 2*(Power(t23,2)*x - t22*t33*x + t12*t33*y + t13*t22*z - t23*(t13*y + t12*z))*(-3*(Power(t13,2) - t11*t33)*(Power(t13,2)*t22 - 2*t12*t13*t23 + Power(t12,2)*t33 + t11*(Power(t23,2) - t22*t33))*(t12*t33*x + Power(t13,2)*y - t11*t33*y + t11*t23*z - t13*(t23*x + t12*z)) + 2*Power(t12*t33*x + Power(t13,2)*y - t11*t33*y + t11*t23*z - t13*(t23*x + t12*z),3)))
            /
            (Sqrt(6)*Power(Pi,0.75)*Power(-(Power(t13,2)*t22) + 2*t12*t13*t23 - t11*Power(t23,2) - Power(t12,2)*t33 + t11*t22*t33,0.25)*Power(Power(t13,2)*t22 - 2*t12*t13*t23 + Power(t12,2)*t33 + t11*(Power(t23,2) - t22*t33),4)));
    }

    template <typename T>
    auto HG02(T x, T y, T z, T t11, T t12, T t13, T t22, T t23, T t33) {
        return 
            exp(-(Power(t23,2)*Power(x,2) + 2*t12*t33*x*y + Power(t13,2)*Power(y,2) - t11*t33*Power(y,2) - 2*t12*t13*y*z + Power(t12,2)*Power(z,2) - 2*t23*(t13*x*y + t12*x*z - t11*y*z) - t22*(t33*Power(x,2) - 2*t13*x*z + t11*Power(z,2)))/(2.*(Power(t13,2)*t22 - 2*t12*t13*t23 + Power(t12,2)*t33 + t11*(Power(t23,2) - t22*t33))))
            * 
            (-(Power(t12,2)*Power(t23,2)*t33) + Power(t12,2)*t22*Power(t33,2) - t11*Power(Power(t23,2) - t22*t33,2) + 2*Power(t23,4)*Power(x,2) - 4*t22*Power(t23,2)*t33*Power(x,2) + 2*Power(t22,2)*Power(t33,2)*Power(x,2) + 4*t12*Power(t23,2)*t33*x*y - 4*t12*t22*Power(t33,2)*x*y + 2*Power(t12,2)*Power(t33,2)*Power(y,2) - 4*t12*Power(t23,3)*x*z + 4*t12*t22*t23*t33*x*z - 4*Power(t12,2)*t23*t33*y*z + 2*Power(t12,2)*Power(t23,2)*Power(z,2) + Power(t13,2)*(2*Power(t23,2)*Power(y,2) - t22*t23*(t23 + 4*y*z) + Power(t22,2)*(t33 + 2*Power(z,2))) + 2*t13*(-2*(Power(t23,2) - t22*t33)*x*(t23*y - t22*z) + t12*(Power(t23,3) + 2*Power(t23,2)*y*z + 2*t22*t33*y*z - t23*(t22*t33 + 2*t33*Power(y,2) + 2*t22*Power(z,2)))))
            /
            (Sqrt(2)*Power(Pi,0.75)*Power(-(Power(t13,2)*t22) + 2*t12*t13*t23 - t11*Power(t23,2) - Power(t12,2)*t33 + t11*t22*t33,0.25)*Power(Power(t13,2)*t22 - 2*t12*t13*t23 + Power(t12,2)*t33 + t11*(Power(t23,2) - t22*t33),2));
    }

    template <typename T>
    auto HG22(T x, T y, T z, T t11, T t12, T t13, T t22, T t23, T t33) {
        return 
            exp(-(Power(t23,2)*Power(x,2) + 2*t12*t33*x*y + Power(t13,2)*Power(y,2) - t11*t33*Power(y,2) - 2*t12*t13*y*z + Power(t12,2)*Power(z,2) - 2*t23*(t13*x*y + t12*x*z - t11*y*z) - t22*(t33*Power(x,2) - 2*t13*x*z + t11*Power(z,2)))/(2.*(Power(t13,2)*t22 - 2*t12*t13*t23 + Power(t12,2)*t33 + t11*(Power(t23,2) - t22*t33))))
            *
            (-(Power(t11,3)*Power(Power(t23,2) - t22*t33,2)*(-(Power(t33,2)*(t22 - 2*Power(y,2))) - 4*t23*t33*y*z + Power(t23,2)*(t33 + 2*Power(z,2)))) + 2*Power(t12,2)*Power(t33,2)*(Power(t12,4)*Power(t33,2) + 2*Power(Power(t23,2) - t22*t33,2)*Power(x,4) + 4*Power(t12,3)*t33*x*(-(t33*y) + t23*z) - 4*t12*(Power(t23,2) - t22*t33)*Power(x,3)*(-(t33*y) + t23*z) + Power(t12,2)*Power(x,2)*(Power(t33,2)*(5*t22 + 2*Power(y,2)) - 4*t23*t33*y*z + Power(t23,2)*(-5*t33 + 2*Power(z,2)))) + Power(t13,6)*(4*Power(t23,2)*Power(y,4) - 4*t22*t23*Power(y,2)*(3*t23 + 2*y*z) - Power(t22,3)*(t33 + 2*Power(z,2)) + Power(t22,2)*(3*Power(t23,2) + 12*t23*y*z + 2*Power(y,2)*(t33 + 2*Power(z,2)))) + t11*t12*t33*(8*Power(Power(t23,2) - t22*t33,2)*Power(x,3)*(-(t33*y) + t23*z) - 8*Power(t12,2)*x*(t33*y - t23*z)*(Power(t33,2)*(t22 + Power(y,2)) - 2*t23*t33*y*z + Power(t23,2)*(-t33 + Power(z,2))) + Power(t12,3)*t33*(Power(t33,2)*(-3*t22 + 10*Power(y,2)) - 20*t23*t33*y*z + Power(t23,2)*(3*t33 + 10*Power(z,2))) - 8*t12*Power(x,2)*(t22*Power(t33,3)*(t22 - 2*Power(y,2)) - 4*Power(t23,3)*t33*y*z + 4*t22*t23*Power(t33,2)*y*z + Power(t23,4)*(t33 + 2*Power(z,2)) - 2*Power(t23,2)*t33*(-(t33*Power(y,2)) + t22*(t33 + Power(z,2))))) - 4*Power(t13,5)*(x*(4*Power(t23,3)*Power(y,3) - Power(t22,3)*t33*z - 2*t22*t23*y*(3*Power(t23,2) + t33*Power(y,2) + 3*t23*y*z) + Power(t22,2)*(3*Power(t23,2)*z + 2*t33*Power(y,2)*z + 2*t23*y*(2*t33 + Power(z,2)))) + t12*(-6*Power(t23,3)*Power(y,2) + 2*t23*t33*Power(y,4) + 2*Power(t22,2)*y*z*(2*t33 + Power(z,2)) + t22*(3*Power(t23,3) + 6*Power(t23,2)*y*z - 2*t33*Power(y,3)*z - 2*t23*Power(y,2)*(2*t33 + Power(z,2))))) + 2*Power(t11,2)*(Power(t33,4)*(-(Power(t22,3)*Power(x,2)) + 2*Power(t12,2)*Power(y,4) - 4*t12*t22*Power(y,2)*(t12 + x*y) + 2*Power(t22,2)*x*y*(4*t12 + x*y)) + 4*t23*Power(t33,3)*(-(Power(t22,2)*Power(x,2)*y) + 2*Power(t12,2)*y*(t22 - Power(y,2)) + t12*t22*x*(-2*t22 + 3*Power(y,2)))*z + Power(t23,6)*Power(x,2)*(t33 + 2*Power(z,2)) - 4*Power(t23,5)*x*z*(t33*x*y + t12*(2*t33 + Power(z,2))) + Power(t23,4)*(2*Power(t33,2)*Power(x,2)*Power(y,2) + 4*t12*t33*x*y*(2*t33 + 3*Power(z,2)) - t22*t33*Power(x,2)*(3*t33 + 4*Power(z,2)) + 2*Power(t12,2)*(2*t33*Power(z,2) + Power(z,4))) - 4*Power(t23,3)*t33*z*(-2*t22*t33*Power(x,2)*y + 2*Power(t12,2)*y*(t33 + Power(z,2)) + t12*x*(3*t33*Power(y,2) - t22*(4*t33 + Power(z,2)))) + Power(t23,2)*Power(t33,2)*(Power(t22,2)*Power(x,2)*(3*t33 + 2*Power(z,2)) + 4*t12*Power(y,2)*(t33*x*y + t12*(t33 + 3*Power(z,2))) - 4*t22*(t33*Power(x,2)*Power(y,2) + Power(t12,2)*Power(z,2) + t12*x*y*(4*t33 + 3*Power(z,2))))) + Power(t13,4)*(-8*t12*x*(6*Power(t23,4)*y - 3*t22*Power(t23,3)*z + t22*t33*y*(-2*t22*t33 + t33*Power(y,2) - 3*t22*Power(z,2)) - t22*t23*z*(2*t22*t33 - 5*t33*Power(y,2) + t22*Power(z,2)) + Power(t23,2)*y*(t22*t33 - 4*t33*Power(y,2) + 2*t22*Power(z,2))) + 2*Power(x,2)*(-(Power(t22,3)*Power(t33,2)) + 12*Power(t23,4)*Power(y,2) - 6*t22*Power(t23,2)*(Power(t23,2) + 2*t33*Power(y,2) + 2*t23*y*z) + Power(t22,2)*(2*Power(t33,2)*Power(y,2) + 8*t23*t33*y*z + Power(t23,2)*(7*t33 + 2*Power(z,2)))) + t11*(-4*Power(t23,2)*Power(y,2)*(3*Power(t23,2) + 2*t33*Power(y,2) - 2*t23*y*z) + Power(t22,3)*t33*(3*t33 + 4*Power(z,2)) + 2*t22*t23*(3*Power(t23,3) + 8*t33*Power(y,3)*z + 2*t23*Power(y,2)*(7*t33 - 4*Power(z,2))) - Power(t22,2)*(Power(t23,2)*(9*t33 - 6*Power(z,2)) + 4*t23*y*z*(5*t33 - 2*Power(z,2)) + 2*t33*Power(y,2)*(3*t33 + 4*Power(z,2)))) + 2*Power(t12,2)*(6*Power(t23,4) + 4*t23*y*z*(t33*Power(y,2) + t22*(5*t33 + Power(z,2))) + Power(t23,2)*(-2*Power(y,2)*(13*t33 + 2*Power(z,2)) + t22*(9*t33 + 6*Power(z,2))) + 2*(Power(t33,2)*Power(y,4) - 2*t22*t33*Power(y,2)*(t33 + 2*Power(z,2)) + Power(t22,2)*(2*t33*Power(z,2) + Power(z,4))))) + 4*t13*(-(t12*t33*(2*t23*Power(Power(t23,2) - t22*t33,2)*Power(x,4) - 2*t12*(Power(t23,2) - t22*t33)*Power(x,3)*(-3*t23*t33*y + Power(t23,2)*z + 2*t22*t33*z) + Power(t12,4)*t33*(3*t23*t33 - 2*t33*y*z + 2*t23*Power(z,2)) - 2*Power(t12,2)*Power(x,2)*(Power(t23,2)*t33*y*z + 3*t22*Power(t33,2)*y*z + Power(t23,3)*(5*t33 + Power(z,2)) - t23*t33*(5*t22*t33 + 2*t33*Power(y,2) + 3*t22*Power(z,2))) + Power(t12,3)*x*(Power(t33,2)*(5*t22 + 2*Power(y,2))*z - 2*t23*t33*y*(5*t33 + 2*Power(z,2)) + Power(t23,2)*(5*t33*z + 2*Power(z,3))))) + Power(t11,2)*(3*Power(t23,6)*x*z + t22*Power(t33,3)*(Power(t22,2)*x + 2*t12*Power(y,3) - 2*t22*y*(2*t12 + x*y))*z - 2*Power(t23,5)*x*y*(2*t33 + Power(z,2)) + Power(t23,2)*t33*z*(6*t12*t33*Power(y,3) + Power(t22,2)*x*(t33 - 2*Power(z,2)) - 2*t22*y*(t33*x*y - 3*t12*Power(z,2))) + Power(t23,4)*z*(2*y*(2*t12*t33 + 2*t33*x*y + t12*Power(z,2)) + t22*(-5*t33*x + 2*x*Power(z,2))) + 2*t23*Power(t33,2)*(-(t12*t33*Power(y,4)) + t22*Power(y,2)*(2*t12*t33 + t33*x*y - 3*t12*Power(z,2)) + 2*Power(t22,2)*(-(t33*x*y) + (t12 + x*y)*Power(z,2))) - 2*Power(t23,3)*(t33*Power(y,2)*(2*t12*t33 + t33*x*y + 3*t12*Power(z,2)) + t22*(-4*Power(t33,2)*x*y + t33*(2*t12 + x*y)*Power(z,2) + t12*Power(z,4)))) + t11*(-2*t23*Power(Power(t23,2) - t22*t33,2)*Power(x,3)*(-(t33*y) + t23*z) + 2*t12*(Power(t23,2) - t22*t33)*Power(x,2)*(-5*Power(t23,2)*t33*y*z - 3*t22*Power(t33,2)*y*z + Power(t23,3)*(2*t33 + Power(z,2)) + t23*t33*(-2*t22*t33 + 4*t33*Power(y,2) + 3*t22*Power(z,2))) + 2*Power(t12,2)*x*(2*t22*Power(t33,3)*(t22 - 2*Power(y,2))*z + Power(t23,3)*t33*y*(-3*t33 + Power(z,2)) - Power(t23,2)*t33*z*(7*t22*t33 + 5*t33*Power(y,2) + 4*t22*Power(z,2)) + t23*Power(t33,2)*y*(3*t22*t33 + 3*t33*Power(y,2) + 8*t22*Power(z,2)) + Power(t23,4)*(5*t33*z + Power(z,3))) + Power(t12,3)*(2*Power(t33,3)*y*(t22 + Power(y,2))*z + 6*Power(t23,2)*t33*y*z*(3*t33 + Power(z,2)) - Power(t23,3)*(3*Power(t33,2) + 8*t33*Power(z,2) + 2*Power(z,4)) + t23*Power(t33,2)*(t22*(3*t33 - 2*Power(z,2)) - 2*Power(y,2)*(5*t33 + 3*Power(z,2)))))) - 4*Power(t13,3)*(2*t23*(Power(t23,2) - t22*t33)*Power(x,3)*(2*Power(t23,2)*y - t22*t33*y - t22*t23*z) - 2*t12*Power(x,2)*(3*Power(t23,5) + 7*t22*Power(t23,2)*t33*y*z - 3*Power(t22,2)*Power(t33,2)*y*z + t22*t23*t33*(-2*t22*t33 + 4*t33*Power(y,2) - 3*t22*Power(z,2)) + Power(t23,3)*(-(t22*t33) - 6*t33*Power(y,2) + t22*Power(z,2))) + 2*Power(t12,2)*x*(-(Power(t23,3)*y*(13*t33 + 2*Power(z,2))) + t23*t33*y*(3*t22*t33 + 3*t33*Power(y,2) - t22*Power(z,2)) + 2*t22*t33*z*(-2*t33*Power(y,2) + t22*(t33 + Power(z,2))) + Power(t23,2)*z*(t33*Power(y,2) + t22*(8*t33 + Power(z,2)))) + Power(t12,3)*(-2*Power(t23,2)*t33*y*z + Power(t23,3)*(7*t33 + 2*Power(z,2)) + 2*t33*y*z*(t33*Power(y,2) + t22*(t33 - Power(z,2))) + t23*(-2*t33*Power(y,2)*(5*t33 + Power(z,2)) + t22*(3*Power(t33,2) + 8*t33*Power(z,2) + 2*Power(z,4)))) + t11*(2*x*(-3*Power(t23,5)*y + 3*Power(t23,4)*Power(y,2)*z + Power(t22,2)*Power(t33,2)*(t22 - 2*Power(y,2))*z + Power(t23,3)*y*(7*t22*t33 - 3*t33*Power(y,2) - 4*t22*Power(z,2)) + t22*t23*t33*y*(-4*t22*t33 + 2*t33*Power(y,2) + t22*Power(z,2)) + t22*Power(t23,2)*z*(-(t22*t33) + 2*t33*Power(y,2) + t22*Power(z,2))) + t12*(3*Power(t23,5) - 6*Power(t23,4)*y*z + 6*Power(t23,2)*t33*y*(-t22 + Power(y,2))*z + Power(t23,3)*(-3*t22*(t33 - 2*Power(z,2)) + 2*Power(y,2)*(t33 - Power(z,2))) - 2*t22*t33*y*z*(-2*t33*Power(y,2) + t22*(4*t33 + Power(z,2))) + 2*t23*(-2*Power(t33,2)*Power(y,4) + 2*t22*t33*Power(y,2)*(2*t33 - Power(z,2)) + Power(t22,2)*(2*t33*Power(z,2) + Power(z,4)))))) + Power(t13,2)*(4*Power(Power(t23,3) - t22*t23*t33,2)*Power(x,4) + 8*t12*t33*(-Power(t23,2) + t22*t33)*Power(x,3)*(-4*Power(t23,2)*y + t22*t33*y + 3*t22*t23*z) - 4*Power(t12,2)*Power(x,2)*(2*Power(t23,3)*t33*y*z + 10*t22*t23*Power(t33,2)*y*z + Power(t23,4)*(13*t33 + 2*Power(z,2)) - Power(t23,2)*t33*(11*t22*t33 + 10*t33*Power(y,2) + 2*t22*Power(z,2)) - 2*t22*Power(t33,2)*(t22*t33 - 2*t33*Power(y,2) + 3*t22*Power(z,2))) + 8*Power(t12,3)*t33*x*(Power(t23,3)*z - Power(t23,2)*y*(11*t33 + 2*Power(z,2)) + t33*y*(t22*t33 + t33*Power(y,2) - 3*t22*Power(z,2)) + t23*z*(9*t22*t33 + t33*Power(y,2) + 3*t22*Power(z,2))) + Power(t12,4)*(-4*t23*t33*y*z*(5*t33 + 2*Power(z,2)) + Power(t33,2)*(3*t22*t33 - 10*t33*Power(y,2) + 10*t22*Power(z,2) + 4*Power(y,2)*Power(z,2)) + Power(t23,2)*(27*Power(t33,2) + 20*t33*Power(z,2) + 4*Power(z,4))) + Power(t11,2)*(3*Power(t23,6) - 12*Power(t23,5)*y*z - Power(t22,2)*Power(t33,2)*(t22 - 2*Power(y,2))*(3*t33 + 2*Power(z,2)) + 4*t22*t23*t33*y*z*(-2*t33*Power(y,2) + t22*(t33 - 2*Power(z,2))) - 8*Power(t23,3)*y*z*(t33*Power(y,2) + t22*(-t33 + Power(z,2))) + Power(t23,4)*(2*Power(y,2)*(7*t33 + 2*Power(z,2)) + t22*(-9*t33 + 6*Power(z,2))) + Power(t23,2)*(4*Power(t33,2)*Power(y,4) - 4*t22*t33*Power(y,2)*(5*t33 - 4*Power(z,2)) + Power(t22,2)*(9*Power(t33,2) - 4*t33*Power(z,2) + 4*Power(z,4)))) + 2*t11*(-4*t12*x*(3*Power(t23,5)*z - Power(t23,3)*t33*(3*t22 + 7*Power(y,2))*z + Power(t23,4)*y*(t33 + 2*Power(z,2)) + t22*Power(t33,2)*y*(4*t22*t33 - 2*t33*Power(y,2) + 3*t22*Power(z,2)) - t22*t23*t33*z*(2*t33*Power(y,2) + 3*t22*Power(z,2)) + Power(t23,2)*t33*y*(-5*t22*t33 + 5*t33*Power(y,2) + 4*t22*Power(z,2))) - 2*(Power(t23,2) - t22*t33)*Power(x,2)*(3*Power(t23,4) + t22*Power(t33,2)*(t22 - 2*Power(y,2)) - 6*Power(t23,3)*y*z - 2*t22*t23*t33*y*z + Power(t23,2)*(-4*t22*t33 + 6*t33*Power(y,2) + 4*t22*Power(z,2))) + Power(t12,2)*(-4*Power(t23,3)*y*z*(8*t33 + Power(z,2)) + Power(t23,4)*(9*t33 + 6*Power(z,2)) + 4*t23*t33*y*z*(-7*t22*t33 + t33*Power(y,2) - 4*t22*Power(z,2)) + 4*Power(t33,2)*(-(t33*Power(y,4)) - Power(t22,2)*Power(z,2) + 2*t22*Power(y,2)*(t33 + Power(z,2))) + Power(t23,2)*(2*t33*Power(y,2)*(11*t33 + 2*Power(z,2)) + t22*(-9*Power(t33,2) + 28*t33*Power(z,2) + 8*Power(z,4)))))))
            /
            (2.*Power(Pi,0.75)*Power(-(Power(t13,2)*t22) + 2*t12*t13*t23 - t11*Power(t23,2) - Power(t12,2)*t33 + t11*t22*t33,0.25)*Power(Power(t13,2)*t22 - 2*t12*t13*t23 + Power(t12,2)*t33 + t11*(Power(t23,2) - t22*t33),4));
    }

    template <typename T>
    auto HG13(T x, T y, T z, T t11, T t12, T t13, T t22, T t23, T t33) {
        return 
            -exp(-(Power(t23,2)*Power(x,2) + 2*t12*t33*x*y + Power(t13,2)*Power(y,2) - t11*t33*Power(y,2) - 2*t12*t13*y*z + Power(t12,2)*Power(z,2) - 2*t23*(t13*x*y + t12*x*z - t11*y*z) - t22*(t33*Power(x,2) - 2*t13*x*z + t11*Power(z,2)))/(2.*(Power(t13,2)*t22 - 2*t12*t13*t23 + Power(t12,2)*t33 + t11*(Power(t23,2) - t22*t33))))
            *
            ((3*(t13*t23 - t12*t33)*(Power(t23,2) - t22*t33)*Power(Power(t13,2)*t22 - 2*t12*t13*t23 + Power(t12,2)*t33 + t11*(Power(t23,2) - t22*t33),2) + 4*Power(-(Power(t23,2)*x) + t22*t33*x + t13*t23*y - t12*t33*y - t13*t22*z + t12*t23*z,3)*(t12*t33*x + Power(t13,2)*y - t11*t33*y + t11*t23*z - t13*(t23*x + t12*z)) - 6*(Power(t23,2) - t22*t33)*(Power(t13,2)*t22 - 2*t12*t13*t23 + Power(t12,2)*t33 + t11*(Power(t23,2) - t22*t33))*(t13*t23*x - t12*t33*x - Power(t13,2)*y + t11*t33*y + t12*t13*z - t11*t23*z)*(Power(t23,2)*x - t22*t33*x + t12*t33*y + t13*t22*z - t23*(t13*y + t12*z)) - 6*(t13*t23 - t12*t33)*(Power(t13,2)*t22 - 2*t12*t13*t23 + Power(t12,2)*t33 + t11*(Power(t23,2) - t22*t33))*Power(Power(t23,2)*x - t22*t33*x + t12*t33*y + t13*t22*z - t23*(t13*y + t12*z),2))
            /
            (Sqrt(6)*Power(Pi,0.75)*Power(-(Power(t13,2)*t22) + 2*t12*t13*t23 - t11*Power(t23,2) - Power(t12,2)*t33 + t11*t22*t33,0.25)*Power(Power(t13,2)*t22 - 2*t12*t13*t23 + Power(t12,2)*t33 + t11*(Power(t23,2) - t22*t33),4)));
    }

    template <typename T>
    auto HG33(T x, T y, T z, T t11, T t12, T t13, T t22, T t23, T t33) {
        return 
            exp(-(Power(t23,2)*Power(x,2) + 2*t12*t33*x*y + Power(t13,2)*Power(y,2) - t11*t33*Power(y,2) - 2*t12*t13*y*z + Power(t12,2)*Power(z,2) - 2*t23*(t13*x*y + t12*x*z - t11*y*z) - t22*(t33*Power(x,2) - 2*t13*x*z + t11*Power(z,2)))/(2.*(Power(t13,2)*t22 - 2*t12*t13*t23 + Power(t12,2)*t33 + t11*(Power(t23,2) - t22*t33))))
            *
            (6*Power(t13*t23 - t12*t33,3)*Power(Power(t13,2)*t22 - 2*t12*t13*t23 + Power(t12,2)*t33 + t11*(Power(t23,2) - t22*t33),3) - 36*Power(t13*t23 - t12*t33,2)*Power(Power(t13,2)*t22 - 2*t12*t13*t23 + Power(t12,2)*t33 + t11*(Power(t23,2) - t22*t33),2)*(-(Power(t23,2)*x) + t22*t33*x + t13*t23*y - t12*t33*y - t13*t22*z + t12*t23*z)*(t12*t33*x + Power(t13,2)*y - t11*t33*y + t11*t23*z - t13*(t23*x + t12*z)) + 9*(t13*t23 - t12*t33)*(Power(t23,2) - t22*t33)*Power(Power(t13,2)*t22 - 2*t12*t13*t23 + Power(t12,2)*t33 + t11*(Power(t23,2) - t22*t33),2)*((Power(t13,2) - t11*t33)*(Power(t13,2)*t22 - 2*t12*t13*t23 + Power(t12,2)*t33 + t11*(Power(t23,2) - t22*t33)) - 2*Power(t12*t33*x + Power(t13,2)*y - t11*t33*y + t11*t23*z - t13*(t23*x + t12*z),2)) - 18*(t13*t23 - t12*t33)*(Power(t13,2)*t22 - 2*t12*t13*t23 + Power(t12,2)*t33 + t11*(Power(t23,2) - t22*t33))*Power(Power(t23,2)*x - t22*t33*x + t12*t33*y + t13*t22*z - t23*(t13*y + t12*z),2)*((Power(t13,2) - t11*t33)*(Power(t13,2)*t22 - 2*t12*t13*t23 + Power(t12,2)*t33 + t11*(Power(t23,2) - t22*t33)) - 2*Power(t12*t33*x + Power(t13,2)*y - t11*t33*y + t11*t23*z - t13*(t23*x + t12*z),2)) - 6*(Power(t23,2) - t22*t33)*(Power(t13,2)*t22 - 2*t12*t13*t23 + Power(t12,2)*t33 + t11*(Power(t23,2) - t22*t33))*(Power(t23,2)*x - t22*t33*x + t12*t33*y + t13*t22*z - t23*(t13*y + t12*z))*(-3*(Power(t13,2) - t11*t33)*(Power(t13,2)*t22 - 2*t12*t13*t23 + Power(t12,2)*t33 + t11*(Power(t23,2) - t22*t33))*(t12*t33*x + Power(t13,2)*y - t11*t33*y + t11*t23*z - t13*(t23*x + t12*z)) + 2*Power(t12*t33*x + Power(t13,2)*y - t11*t33*y + t11*t23*z - t13*(t23*x + t12*z),3)) + 4*Power(Power(t23,2)*x - t22*t33*x + t12*t33*y + t13*t22*z - t23*(t13*y + t12*z),3)*(-3*(Power(t13,2) - t11*t33)*(Power(t13,2)*t22 - 2*t12*t13*t23 + Power(t12,2)*t33 + t11*(Power(t23,2) - t22*t33))*(t12*t33*x + Power(t13,2)*y - t11*t33*y + t11*t23*z - t13*(t23*x + t12*z)) + 2*Power(t12*t33*x + Power(t13,2)*y - t11*t33*y + t11*t23*z - t13*(t23*x + t12*z),3)))
            /
            (6.*Power(Pi,0.75)*Power(-(Power(t13,2)*t22) + 2*t12*t13*t23 - t11*Power(t23,2) - Power(t12,2)*t33 + t11*t22*t33,0.25)*Power(Power(t13,2)*t22 - 2*t12*t13*t23 + Power(t12,2)*t33 + t11*(Power(t23,2) - t22*t33),6));
    }
}

MTS_NAMESPACE_BEGIN

// static constexpr int HGCoefs = 4;
// static const Float hg_coeffs_sphere[HGCoefs][HGCoefs] = {
//     { 1.02324,   0.0,    0.458325,   0.0,},
//     { 0.0,       0.0,    0.0,        0.0,},
//     { 0.45832,   0.0,    0.187563,   0.0,},
//     { 0.0,       0.0,    0.0,        0.0,},
// };
// static const Float hg_coeffs_cylinder[HGCoefs][HGCoefs] = {
//     { 1.1725,    0.0,    0.7001,     0.0,},
//     { 0.0,       0.0,    0.0,        0.0,},
//     { 0.5236,    0.0,    0.2068,     0.0,},
//     { 0.0,       0.0,    0.0,        0.0,},
// };
static constexpr int HGCoefs = 1;
static const Float hg_coeffs_sphere[HGCoefs][HGCoefs] = {
    { 1.0 },
};
static const Float hg_coeffs_cylinder[HGCoefs][HGCoefs] = {
    { 1.0 },
};

#define ForEachHGCoeff(n,m) for (unsigned n=0;n<HGCoefs;++n)for (unsigned m=(n%2);m<HGCoefs;m+=2)

inline Float HG(unsigned n, unsigned m, const Vector3 &xi, const Matrix3x3 &Theta) {
#ifdef MTS_DEBUG
    assert(n<4 && m<4);
#endif
    
    if (((n+m)%2)==1)
        return 0;

    const auto x = xi[0], y = xi[1], z = xi[2];
    const auto t11 = Theta(0,0), t12 = Theta(0,1), t13 = Theta(0,2), t22 = Theta(1,1), t23 = Theta(1,2), t33 = Theta(2,2);

    if (n==0&&m==0)
        return _HG_details::HG00(x,y,z,t11,t12,t13,t22,t23,t33);
    if (n==2&&m==0)
        return _HG_details::HG20(x,y,z,t11,t12,t13,t22,t23,t33);
    if (n==1&&m==1)
        return _HG_details::HG11(x,y,z,t11,t12,t13,t22,t23,t33);
    if (n==0&&m==2)
        return _HG_details::HG02(x,y,z,t11,t12,t13,t22,t23,t33);
    if (n==2&&m==2)
        return _HG_details::HG22(x,y,z,t11,t12,t13,t22,t23,t33);
    if (n==3&&m==1)
        return _HG_details::HG31(x,y,z,t11,t12,t13,t22,t23,t33);
    if (n==1&&m==3)
        return _HG_details::HG13(x,y,z,t11,t12,t13,t22,t23,t33);
    if (n==3&&m==3)
        return _HG_details::HG33(x,y,z,t11,t12,t13,t22,t23,t33);
    return 0;
}

MTS_NAMESPACE_END
